#include <stdio.h>
#include "SDL.h"

#include "charactron.h"

void charactron_8bpp_color_cache(charactron *c)
{
    SDL_Color fg,bg;
    fg=c->font->format->palette->colors[1];
    bg=c->font->format->palette->colors[0];
    c->fg = SDL_MapRGB(c->output->format, fg.r, fg.g, fg.b);
    c->bg = SDL_MapRGB(c->output->format, bg.r, bg.g, bg.b);
}

int charactron_set_color(charactron *c, int i, SDL_Color cl)
{
  if(c->font->format->BitsPerPixel != 8)
    return(-1);
  if((i<0) || (i>= c->font->format->palette->ncolors))
    return(-1);

  if(SDL_SetColors(c->font,&cl,i,1)<0)
    return(-1);

  charactron_8bpp_color_cache(c);
  return(0);  
}

void charactron_rect(const charactron *c, Uint8 letter, SDL_Rect *r)
{
  r->w=CHARACTRON_FONT_W(c);
  r->h=CHARACTRON_FONT_H(c);
  r->y=(letter>>4)*r->h;
  r->x=(letter&0x0f)*r->w;
}

void charactron_free(charactron *c)
{
  if(c != NULL)
  {
    if(c->font != NULL)
      SDL_FreeSurface(c->font);

    if((c->output != NULL)&&(c->output != SDL_GetVideoSurface()))
      SDL_FreeSurface(c->output);

    free(c);
  }
}

charactron *charactron_load(SDL_RWops *img, SDL_Surface *out, int autoclose)
{
  charactron *c=NULL;

  if(img==NULL)
      goto CHARACTRON_LOAD_ERR;

  if(out == NULL)
  {
    out=SDL_GetVideoSurface();
    if(out == NULL)
      goto CHARACTRON_LOAD_ERR;
  }

  c=(charactron *)malloc(sizeof(charactron));
  if(c==NULL)
    goto CHARACTRON_LOAD_ERR;

  c->output=out;
  //c->font=IMG_Load_RW(img,autoclose);
  c->font=SDL_LoadBMP_RW(img,autoclose);

  if(c->font == NULL)
    goto CHARACTRON_LOAD_ERR;

  c->maxx=c->output->w / CHARACTRON_FONT_W(c);
  c->maxy=c->output->h / CHARACTRON_FONT_H(c);
  c->flag=CH_ENABLE;

  if(c->font->format->BitsPerPixel == 8)
    charactron_8bpp_color_cache(c);
  else
  { 
    c->fg=SDL_MapRGB(c->output->format,255,255,255);
    c->bg=SDL_MapRGB(c->output->format,  0,  0,  0);
  }

  charactron_formfeed(c);

  return(c);

CHARACTRON_LOAD_ERR:
  charactron_free(c);
  return(NULL);
}

int charactron_draw_letter(charactron *c, Uint8 letter)
{
  SDL_Rect src,dest;
  if(CHARACTRON_W_FREE(c) == 0)
    return(-1);

  if(CHARACTRON_ENABLED(c))
  {
    charactron_rect(c,16+1,&dest);
    charactron_rect(c,letter,&src);

    dest.x*=c->x;
    dest.y*=c->y;
    SDL_BlitSurface(c->font,&src,c->output,&dest);
  }

  c->x++;

  return(0);
}

int charactron_draw_data(charactron *c, const char *str, long len)
{
  int n=0;
  SDL_Rect dest;
  if(len < 0) len=strlen(str);

  if(len > CHARACTRON_W_FREE(c))
    return(-1);

  if(CHARACTRON_ENABLED(c))
  {
    charactron_rect(c,16+1,&dest);
    dest.x*=c->x;
    dest.y*=c->y;

    while(n<len)
    {
      SDL_Rect src;
      charactron_rect(c,str[n],&src);
      SDL_BlitSurface(c->font,&src,c->output,&dest);

      dest.x+=CHARACTRON_FONT_W(c);
      n++;
    }
  }

  c->x+=len;

  return(0);
}

int charactron_linefeed(charactron *c)
{
  if(c->y >= (c->maxy - 1))
    return(-1);

  c->y++;
  c->x=0;
  return(0);
}

int charactron_formfeed(charactron *c)
{
  if(CHARACTRON_ENABLED(c))
    SDL_FillRect(c->output,NULL,c->bg);

  c->x=0;
  c->y=0;
}
